<?php

namespace Drupal\Tests\state_machine\Functional;

use Drupal\Core\Entity\Entity\EntityViewDisplay;
use Drupal\entity_test\Entity\EntityTestWithBundle;
use Drupal\Tests\BrowserTestBase;

/**
 * Tests the state transition form.
 *
 * @group state_machine
 */
class StateTransitionFormTest extends BrowserTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'entity_test',
    'options',
    'state_machine',
    'state_machine_test',
    'views',
  ];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * {@inheritdoc}
   */
  protected function setUp() {
    parent::setUp();

    $user = $this->drupalCreateUser(['administer entity_test content', 'view test entity']);
    $this->drupalLogin($user);
  }

  /**
   * Tests the transition form.
   */
  public function testForm() {
    $first_entity = EntityTestWithBundle::create([
      'type' => 'first',
      'name' => 'First',
    ]);
    $first_entity->save();

    $second_entity = EntityTestWithBundle::create([
      'type' => 'second',
      'name' => 'Second',
      'field_state' => 'validation',
    ]);
    $second_entity->save();

    $this->drupalGet('/state-machine-test');
    $this->assertSession()->pageTextContains('First');
    $this->assertSession()->pageTextContains('New');
    $buttons = $this->xpath('//form[@id="state-machine-transition-form-entity-test-with-bundle-field-state-1"]/div/input');
    $this->assertCount(2, $buttons);
    $this->assertEquals('Create', $buttons[0]->getValue());
    $this->assertEquals('Cancel', $buttons[1]->getValue());

    $this->assertSession()->pageTextContains('Second');
    $this->assertSession()->pageTextContains('Validation');
    $buttons = $this->xpath('//form[@id="state-machine-transition-form-entity-test-with-bundle-field-state-2"]/div/input');
    $this->assertCount(1, $buttons);
    $this->assertEquals('Validate', $buttons[0]->getValue());

    // Click the Validate button.
    $buttons[0]->click();
    $this->assertSession()->pageTextContains('First');
    $this->assertSession()->pageTextContains('New');
    $buttons = $this->xpath('//form[@id="state-machine-transition-form-entity-test-with-bundle-field-state-1"]/div/input');
    $this->assertCount(2, $buttons);
    $this->assertEquals('Create', $buttons[0]->getValue());
    $this->assertEquals('Cancel', $buttons[1]->getValue());
    $this->assertSession()->buttonExists('Create');

    $this->assertSession()->pageTextContains('Second');
    $this->assertSession()->pageTextContains('Fulfillment');
    $buttons = $this->xpath('//form[@id="state-machine-transition-form-entity-test-with-bundle-field-state-2"]/div/input');
    $this->assertCount(1, $buttons);
    $this->assertEquals('Fulfill', $buttons[0]->getValue());
  }

  /**
   * Tests the confirmation form.
   */
  public function testConfirmationForm() {
    $view_display = EntityViewDisplay::create([
      'targetEntityType' => 'entity_test_with_bundle',
      'bundle' => 'first',
      'mode' => 'default',
      'status' => TRUE,
    ]);
    $view_display->setComponent('field_state', [
      'type' => 'state_transition_form',
      'settings' => [
        'require_confirmation' => TRUE,
      ],
    ]);
    $view_display->save();
    $first_entity = EntityTestWithBundle::create([
      'type' => 'first',
      'name' => 'First',
    ]);
    $first_entity->save();
    $this->drupalGet($first_entity->toUrl('canonical'));
    $this->assertSession()->pageTextContains('First');
    $buttons = $this->xpath('//form[@id="state-machine-transition-form-entity-test-with-bundle-field-state-1"]/div/input');
    $this->assertCount(2, $buttons);
    $this->assertEquals('Create', $buttons[0]->getValue());
    $this->assertEquals('Cancel', $buttons[1]->getValue());
    $this->assertSession()->buttonExists('Create');
    // Click the Create button.
    $buttons[0]->click();
    $this->assertSession()->pageTextContains('Are you sure you want to create First?');
    $this->assertSession()->pageTextContains('This action cannot be undone.');
    $this->assertSession()->linkExists('Cancel');
    $this->submitForm([], t('Create'));

    $buttons = $this->xpath('//form[@id="state-machine-transition-form-entity-test-with-bundle-field-state-1"]/div/input');
    $this->assertCount(1, $buttons);
    $this->assertEquals('Fulfill', $buttons[0]->getValue());
  }

}
